@@ -10,7 +10,7 @@ BEGIN {
   $_scode = inet_aton('127.0.0.0');
 }
 
-$VERSION = do { my @r = (q$Revision: 0.15 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 0.16 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 use AutoLoader 'AUTOLOAD';
 
@@ -626,9 +626,13 @@ sub dns_ptr {
   ($off,my($name,$type,$class)) = $get->Question($bp,$off);
   return undef unless $class == C_IN;
 
-  ($off,$name,$type,$class,my($ttl,$rdlength,$host)) =
+  while (1) {
+    ($off,$name,$type,$class,my($ttl,$rdlength,$host)) =
 	$get->next($bp,$off);
-  return undef unless $type == T_PTR;
+    last if $type == T_PTR;
+    next if $type == C_NAME;
+    return undef;			# not a PTR or CNAME record
+  }
   ($name,$type,$class,$host) = $parse->PTR($name,$type,$class,$host);
   return $host;
 }
